/*
  BLE String Example

  Creates two string characteristics, one for receive and one for transmit.
  When a central connects, it listens for incoming on the rxCharacteristic,
  and sends incoming Serial out on the txCharacteristic.

  rewritten to use event handlers Mar 2022.
  
  Thanks to Ami Mehta for the suggestion to write this.

  created 16 Mar 2021
  modified 22 Mar 2022
  by Tom Igoe
*/

#include <ArduinoBLE.h>

int flag = 0;
String sending1 = "";
String sending2 = "";
String receiving1 = "";
int flow = 0;
const byte ledPin = 4;
const byte interruptPin = 2;
volatile byte state = LOW;
unsigned int cont = 0;

// set the size of the incoming and outgoing strings. Max 512:
const int characteristicSize = 128;

// create service and characteristics:
BLEService stringService("7def8317-7300-4ee6-8849-46face74ca2a");
BLEStringCharacteristic txChar1("7def8317-7301-4ee6-8849-46face74ca2a",
    BLERead | BLENotify, characteristicSize);
BLEStringCharacteristic txChar2("7def8317-7302-4ee6-8849-46face74ca2a",
    BLERead | BLENotify, characteristicSize);
BLEStringCharacteristic rxChar1("7def8317-7303-4ee6-8849-46face74ca2a",
    BLEWrite, characteristicSize);

void setup() {
  // initialize serial and BLE:
  Serial.begin(9600);

  if (!BLE.begin()) {
    Serial.println("starting BLE failed!");
    while (true);
  }
  // use the builtin LED as a connect/disconnect handler:
  pinMode(LED_BUILTIN, OUTPUT);
  pinMode(ledPin, OUTPUT);
  pinMode(interruptPin, INPUT_PULLUP);
  attachInterrupt(digitalPinToInterrupt(interruptPin), blink, RISING);

  // set the local name peripheral advertises:
  BLE.setLocalName("BLE_String");
  // set the UUID for the service this peripheral advertises:
  BLE.setAdvertisedService(stringService);

  // add the characteristics to the service:
  stringService.addCharacteristic(txChar1);
  stringService.addCharacteristic(txChar2);
  stringService.addCharacteristic(rxChar1);

  // assign event handlers for connected, disconnected to peripheral
  BLE.setEventHandler(BLEConnected, connectHandler);
  BLE.setEventHandler(BLEDisconnected, disconnectHandler);

  // assign event handler for rx characteristic:
  rxChar1.setEventHandler(BLEWritten, incomingDataHandler);

  // add the service and set a value for the characteristic:
  BLE.addService(stringService);
  // start advertising
  BLE.advertise();
}

void loop() {
  // Listen for events:
  BLE.poll();

  //if a serial string comes in, set it in the txCharacteristic:

   flow = flow + 1;
   sending1 = String(cont);
   sending2 = String(flow);
   txChar1.setValue(sending1);
   txChar2.setValue(sending2);
   rxChar1.setValue(receiving1);
   Serial.print("testo1:   ");
   Serial.println(sending1);
   Serial.print("testo2:   ");
   Serial.println(sending2);
   Serial.print("testo3:   ");
   Serial.println(receiving1);     
delay(150);
}

void connectHandler(BLEDevice central) {
  // central connected event handler
  Serial.print("Connected event, central: ");
  Serial.println(central.address());
  digitalWrite(LED_BUILTIN, HIGH);
}

void disconnectHandler(BLEDevice central) {
  // central disconnected event handler
  Serial.print("Disconnected event, central: ");
  Serial.println(central.address());
  digitalWrite(LED_BUILTIN, LOW);
}

void incomingDataHandler(BLEDevice central, BLECharacteristic characteristic) {
  // central wrote new value to characteristic, update LED
  Serial.print("Characteristic event, written: ");
  Serial.println(rxChar1.value());
}

void blink() {
  cont = cont + 1;
  digitalWrite(ledPin, state);
  state = !state;
  }
